#ifndef _OKAN_MOS_H
#define _OKAN_MOS_H

#include "OkanMisc.h"

#include <string>

#define NEAR_NEIG 0
#define BILINEAR 1
#define BICUBIC 2

using namespace std;

/**
 * Image Mosaicking class
 *
 * @author Yunus OLGUN
 */
class OkanMos
{
	public:
/**
 * Private member which keeps base image and last image added to mosaic
 */
	IplImage* image[2];

/**
 * Private member which keeps world file information
 */
	double line[6];

/**
 * Private member which keeps mask of mosaic. Valid pixels are 1 , incalids are 0.
 */
	uchar **realMask;

/**
 * Private member which keeps homography matix for transormations. 
 * image[0] * homMat[0] => image[1]
 * image[1] * homMat[1] => image[0]
 */
	CvMat* homMat[2];

/**
 * Private member used for removing illumination differencies.
 * newValue = image[1] * shadeMul + shadeAdd
 */
	double shadeMul[3];

/**
 * Private member used for removing illumination differencies.
 * newValue = image[1] * shadeMul + shadeAdd
 */
	double shadeAdd[3];

/**
 * Private member function to calculate shadeAdd and shadeMul fields for removing illumination differencies.
 * @param i0 first image
 * @param i1 second image
 * @param m0 homography matrix to be used in transformation from first image to second image
 * @param m1 homography matrix to be used in transformation from second image to first image
 */
	void removeShade( IplImage* &i0 , IplImage* &i1 , CvMat* &m0 , CvMat* &m1 );

/**
 * Core function of this class. It creates a new image[0] from current image[0] and image[1]. It uses shadeAdd and shadeMul. It works according to   
 * interpolationType
 * @param interpolationType It is used for interpolation type. use 0 or 1.
 * 		0 = Nearest Neighboorhod
 * 		1 = Bilinear interpolation
 */
	void makeMosaic( int interpolationType , uchar** im1Mask = NULL );

/**
 * It rounds x to an integer between 0 and 255. It is an inline function.
 * @param x It will be converted
 * @return Round of x
 */
	uchar makeUImagePixel( double x);

	public:

/**
 * Constructor. It makes everything NULL
 */
	OkanMos();

/**
 * Destructor. It only releases homMat field. You should release image[0] and realMask manually by using getMosaic
 */
	~OkanMos();

/**
 * Get method of image[0] and realMask
 * @param im It will keep pointer to image after function. It is used as a return value
 * @param mask It will keep pointer to mask after function. It is used as a return value
 */	
	void getMosaic( IplImage ** im , uchar *** mask , double imLine[6] = NULL );

/**
 * Sets base image. Base image is image[0]. It takes a copy of i0. According to i0's size it allocates memory for realMask.
 * @param i0 image pointer of base image
 */	
	void setBaseImage( IplImage* i0 , uchar** i0mask = NULL  , double i0line[6] = NULL );

/**
 * Adds a new image to the mosaic. It does not take copy of suc image. It makes a shallow copy. Please do not deallocate suc image before you
 * add a new image to the mosaic
 * @param suc new image. Do not deallocate it before calling another addSucImage
 * @param mbase Homography matrix which transforms image[0] to suc if image[1] is NULL. Otherwise it transforms image[1] to suc
 * @param msuc Homography matrix which transforms suc to image[0] if image[1] is NULL. Otherwise it transforms suc to image[1]
 * @param interpolationType It is used for interpolation type. use 0 or 1.
 * 		0 = Nearest Neighboorhod
 * 		1 = Bilinear interpolation
 */
	void addSucImage( IplImage* suc , CvMat* mbase , CvMat* msuc , int interpolationType , uchar** i1mask = NULL );
};

#endif

