#ifndef _OKAN_REG_H_
#define _OKAN_REG_H

#include "OkanMisc.h"
#include "highgui.h"

#include <iostream>
#include <string>
#include <vector>

/**
 * Image Registration class
 *
 * @author Yunus OLGUN
 */	
class OkanReg
{
	public:
/**
 * Private IPLImage* array which keeps registered images.
 */
	IplImage* image[2];
	
/**
 * Private IPLImage* array which keeps gray level of registered images.
 */
	IplImage* gray[2];
	
/**
 * Private CVPoint2D32f* array which keeps corner points of two images.
 */
	CvPoint2D32f *corner[2];
	
/**
 * Private int array which keeps size of corner points of two images.
 */
	int cornerSize[2];
	
/**
 * Private CvPoint2D32f vector which is used for putative matches of two images.
 */
	vector< CvPoint2D32f > matches[2];
	
/**
 * Private CVMat* array which keeps best homography matrix of two images.
 * bestMat[0] means image[0] * bestMat[0] => image[1]
 */
	CvMat* bestMat[2];

/**
 * Private int which is used for estimating value of bestMat homography matrix.
 * It means that how many of putative matches corresponds to best homography matrix.
 */
	int bestPoint;
	
/**
 * Private float which is used for mean square error of best homography matrix
 */
	float bestMse;

/**
 * Private member function to calculate normalized cross correlation of two points.
 * @param x0 x coordinate of first point
 * @param y0 y coordinate of first point
 * @param x1 x coordinate of second point
 * @param y1 y coordinate of second point
 * @param n window size for calculating mean and standard deviation of a corner
 * @param mean0 mean of first point
 * @param std0 standard deviation of first point
 * @param mean1 mean of second point
 * @param std1 standard deviation of second point
 * @return normalized cross correlation of two points
 */	
	double norCrosCorr( int x0 , int y0 , int x1 , int y1 , int n , double mean0 , double std0 , double mean1 , double std1 );

/**
 * Finds homography matrix for one subset of putative matches.
 * @param ids It contains 4 integer which are indexes of putative matches
 * @param h A CvMat pointer to be filled by this function. It is used as return value.
 */
	void findHomography( int * ids , CvMat* h );

/**
 * A private function to calculate quality of a homography matrix
 * @param h the homography matrix
 * @param errTol error tolerance while calculating quality of a homography matrix
 * @param mse mean square error of homography matrix. It is used as a return value
 * @return quality of homography matrix
 */	
	int checkHomographyQuality( CvMat* h , float errTol , float & mse );
	

	public:
/**
 * Constructor of OkanReg class. It simply makes all values 0 and all pointers NULL.
 */
	OkanReg();
	
/**
 * Destructor of OkanReg class. It releases all data it has. Be carefull about that, it does not
 * releases image field of the class.
 * @see destroyImage()
 */
	~OkanReg();

/**
 * It does same work of a destructor.
 * @see ~OkanReg()
 * @see clearReg()
 */
	void clearReg();
	
/**
 * It releases all data of the image which has the id. It is used for destructor and clearReg also.
 * @param id id of image
 * @see ~OkanReg()
 * @see ~clearReg()
 */	
	void destroyImage( int id );

/**
 * It sets im as a new image for registration process. It does not copy all image. It only copy the pointer.
 * It releases all data for previous image named id. It calls destroyImage() for this process.
 * @param im pointer of new image
 * @param id the of image to be setted. It should 0 or 1
 * @see destroyImage()
 */
	void setImage( IplImage* im , int id );
	
/**
 * @todo DO NOT USE THIS FUNCTION. While removing it do not forget to remove highgui.h from includes!!!
 */
	void setImage( string im , int id );
	
/**
 * Get method of image field
 * @param id the id of the image which will be returned
 * @return pointer to the this image
 */
	IplImage* getImage( int id );

/**
 * sets region of interest for the image. Give a coordination of rectangle. Be carefull about that it affects every computation.
 * @param id image id
 * @param x1 x coordinate of top-left of rectangle
 * @param y1 y coordinate of top-left of rectangle
 * @param x2 width of rectangle
 * @param y2 heigth of rectangle
 */
	void setImageROI( int id , int x1 , int y1 , int x2 , int y2 );

/**
 * Get method for region of interest of image
 * @param id image id
 * @return CvRect which contains rectangle for region of interest
 */
	CvRect getImageROI( int id );

/**
 * Removes region of interest from image.
 * @param id image id
 */
	void removeImageROI( int id );

/**
 * Returns size of corners
 * @param id image id
 * @return size of corners
 */
	int getCornerSize( int id );

/**
 * Get method of corner field.
 * @param id image id
 * @return return an array which contains corners.
 */
	CvPoint2D32f* getCorner( int id );
	
/**
 * Clears putative matches of two images. Because they are related with each other two of them will be removed at once.
 */
	void clearMatches();

/**
 * Adds a putative match to matches field.
 * @param p1 point for first image
 * @param p2 point for second image
 */
	void addMatch( CvPoint2D32f p1 , CvPoint2D32f p2 );
	
/**
 * Removes the putative match from matches field.
 * @param id index of the putative match
 */
	void eraseMatch( int id );

/**
 * Get method of images
 * @param id image id
 * @return vector of putative matches
 */
	vector< CvPoint2D32f > & getMatches( int id );

/**
 * Get method of best homography matrix
 * @param id image id
 * @return best homography matrix
 */
	CvMat* getBestMat( int id );

/**
 * Returns the number of putative matches which correspond to best homography matrix
 * @return the number of putative matches which correspond to best homography matrix
 */
	int getBestPoint();

/**
 * Returns mean square error of best homography matrix
 * @return mean square error of best homography matrix
 */
	float getBestMse();
	
/**
 * Finds corners of the given image and pushes them into the corner field and sets cornerSize. ROI is effective at this.
 * @param id image id
 * @param n how many points should be found. If n is equal to 0, then thresh is ignored.
 * @param thresh quality level for determining weak corener points.
 * @param minDistance minimum distance between two corner points. According to this weaker ones will be eliminated from result.
 * @param blockSize block size of image. Default value is 3.
 * @param apertureSize aperture size for process. Default value is 3.
 * @param k a parameter for the corner detection algorithm. Default value is 0.04
 * @return selected weakest corner's quality level
 */
	double findHarris( int id  , int n , double thresh , double minDistance , int blockSize = 3 , int apertureSize = 3 , double k = 0.04 );

/**
 * Finds putative matches of two images and pushes them into the matches field. If cornerSize is 0 it throws an exception. ROI is not effective
 * @param n window size for calculating mean and standard deviation of a corner.
 * @param d maximum distance between two corners to be a putative match.
 * @param thresh thresh value for eliminating weak putative matches.
 * @return size of found putative matches
 */
	int findPutMatch( int n , int d = 0 , double thresh = -2 );

/**
 * Finds best homography matrix according to RANSAC algorithm. If size of 4-combinations is less than iter it tries every possible one.
 * Otherwise it applies RANSAC algorithm
 * @param iter maximum number of iterations for RANSAC algorithm
 * @param errTol error tolerance for homography matrix. It is used in determining quality of homography matrix. If the difference between
 * matches is smaller than this errTol variable, homography matrix's quality increases 1 point
 * @return If successfull it return true, otherwise false
 */
	bool findRANSACHomography( int iter , float errTol );
};

#endif
