#ifndef _CONTENT_H_
#define _CONTENT_H_
#include <string>

#include "globals.h"
#include "imageHandle.h"
#include "OkanMisc.h"

#include <cv.h>
#include <highgui.h>

using namespace std;

enum ContentType {
  Image2D, DEM, OrthophotoType, Mosaic, SuperResolution, CameraType, VideoType, UnknownType
};
enum AddressType {
  RelativeAddress, AbsAddress
};
class Content {
public:
	string address;
	string name;
	string comment;
	ContentType contentType;
	AddressType addressType;
	int modified;
	Content(string name, string address, string comment, ContentType contentType, AddressType addressType = RelativeAddress) {
		this->address	= address;
		this->name = name;
		this->comment = comment;
		this->contentType = contentType;
		this->addressType = addressType;
		modified = 0;
	}
	virtual ~Content() {}
};

class Image : public Content {
  friend class MosaickerPanel;
  friend class OPGPanel;
  friend class ImageFactory;
  friend class ImageProcessor;
 protected:
  ImageHandle* imHandle;
  string assocCamera;
  string assocDem;
  double* worldData;
 public:
  unsigned char** mask;
 public:
 Image(string name, string address, string comment, string worldFile, AddressType addressType = RelativeAddress) : Content (name, address, comment, Image2D, RelativeAddress) {
    imHandle = NULL;
    assocCamera = "";
    assocDem = "";
    mask = NULL;
    worldData = NULL;
  }
  virtual ~Image() {
    delete imHandle;
  }
  int openImage(int mode=IM_READONLY);
  int closeImage();

  unsigned char* copyRawData();
  unsigned char* copyOriginalRawData() {
    return imHandle->copyOriginalRawData();
  }
  void getRawData(unsigned char**, int*, Roi*);
  unsigned char* getOriginalRawData();
  int getWidth();
  int getHeight();
  int getBpp();
  IplImage* getOrigIplHandle();
  string getCamera() {
    return assocCamera;
  }
  void setCamera(string camera) {
    assocCamera = camera;
  }
  void setDem(string assocDem) {
    this->assocDem = assocDem;
  }

  string getDem() {
    return assocDem;
  }
  virtual int clonePhysical(string);
  IplImage* getIplHandle();
  double* getWorldData() {
    return worldData;
  }
  void setWorldData(double* newWorldData) {
    worldData = newWorldData;
  }
};

class Camera : public Content {
 public:
  double phi, kappa, omega, pSize, x, y, z, f, px, py;
  Camera(string name, string comment) : Content(name, string(""), comment, CameraType) {
  }

};

class Orthophoto : public Image {
public:
  Orthophoto(string name, string address, string comment, string worldFile)
    : Image(name, address, comment, worldFile) {
    contentType = OrthophotoType;
  }
  int clonePhysical(string);
};

class Mosaick : public Image {
 public:
  Mosaick(string name, string address, string comment, string worldFile) 
    : Image(name, address, comment, worldFile) {
    contentType = Mosaic;
  }

  int clonePhysical(string);
};

class Dem : public Image {
 private:
  unsigned int width, height;
  double* heightTable;
  double minVal, maxVal;
 public:
 Dem(double* heightTable, int width, int height, string name, string address, string comment) :
  Image(name, "INMEM", comment, "NW") {
    contentType = DEM;
    maxVal = DBL_MIN;
    minVal = DBL_MAX;
    for(unsigned int i = 0; i < width * height; i++) {
      if(heightTable[i] > maxVal) {
	maxVal = heightTable[i];
      }
      if(heightTable[i] < minVal) {
	minVal = heightTable[i];
      }
    }
    ImageHandle* newHandle = new ImageHandle;
    newHandle->width = width;
    newHandle->height = height;
    newHandle->bpp = 3;
    newHandle->imagePath = "";
    newHandle->rawData = new unsigned char[width * height * 3];
    newHandle->origData = NULL;
    newHandle->dataset = NULL;
    assocCamera = "";
    imHandle = newHandle;
    double range = maxVal - minVal;
    unsigned int c = 0;
    
    for(unsigned int i = 0; i < 3 * width * height; i += 3, c++) {
      unsigned char compVal = (((heightTable[c] - minVal) * 255) / range);
      newHandle->rawData[i] = compVal;
      newHandle->rawData[i + 1] = compVal;
      newHandle->rawData[i + 2] = compVal;
    }
  
  
  this->width = width;
  this->height = height;
  this->heightTable = heightTable;
}

double* getHeightTable() {
    return heightTable;
  }
  
  int getWidth() {
    return width;
  }

  int getHeight() {
    return height;
  }
};

class Video : public Content {
private:
  CvCapture* videoCapture;
public:
  Video(string name, string address, string comment) :
    Content(name, address, comment, VideoType) {
    videoCapture = cvCaptureFromFile(address.c_str());
  }

  IplImage* grabFrame(int step) {
    for(unsigned int i = 0; i < step; i++) {
      cvGrabFrame(videoCapture);
    }
    cvNamedWindow("window1");
    IplImage* newIm = cvRetrieveFrame(videoCapture);
    cvShowImage("window1", newIm);
    return cvRetrieveFrame(videoCapture);
  }
};
#endif
