#include <gdal_priv.h>
#include <string>

#include "imageHandle.h"
#include "globals.h"
#include <cv.h>

using namespace std;

ImageHandle::ImageHandle() {
  origData = NULL;
  dataset = NULL;
  rawData = NULL;
  width = height = bpp = -1;
}

int ImageHandle::loadImage(string imagePath, int mode) {
  int imMode;
  if(mode == IM_READONLY)
    imMode = GA_ReadOnly;
  
  assert(imMode == GA_ReadOnly);

  dataset = (GDALDataset*)GDALOpen(imagePath.c_str(), GA_ReadOnly);
  if(!dataset)
    return 0;
  rawData = copyRawImageData();
  width = dataset->GetRasterXSize();
  height = dataset->GetRasterYSize();
  bpp = 3;

  origData = copyRawImageData();
  
  return 1;
}

int ImageHandle::releaseImage() {
  if(dataset) {
    GDALClose(dataset);
    return 1;
  }
  return 0;
}
int ImageHandle::getWidth() {
  if(dataset)
    return dataset->GetRasterXSize();
  if(rawData)
    return width;
  return -1;
}

int ImageHandle::getHeight() {
  if(dataset)
    return dataset->GetRasterYSize();
  if(rawData)
    return height;
  return -1;
}

int ImageHandle::getBpp() {
  if(dataset)
    return 3 ; //dataset->GetRasterCount();
  if(rawData)
    return 3; //bpp;
  return -1;
}

IplImage* ImageHandle::getIplHandle() {
  if(!dataset && !rawData)
    return NULL;
  IplImage* iplHandle = cvCreateImage(cvSize(getWidth(), getHeight()), IPL_DEPTH_8U, 3);
  unsigned char* rData;
  //if(dataset) {
  //  rData = copyRawImageData();
  //}
  //else {
    rData = rawData;
    //}
  int width = getWidth();
  int height = getHeight();

  unsigned char* rawData2 = new unsigned char[width*height*3];
  unsigned char* rp = rData;
  for(unsigned int i = 0; i < height; i++, rp += width * 3) {
    for(unsigned int k = 0; k < width; k++) {
      rawData2[(i*width*3) + 3 * k] = rp[3 * k + 2];
      rawData2[(i*width*3) + 3 * k + 1] = rp[3 * k + 1];
      rawData2[(i*width*3) + 3 * k + 2] = rp[3 * k];
    }
  }

  cvSetData(iplHandle, rawData2, getWidth() * 3);
  //IplImage* iplHandeC = cvCloneImage(iplHandle);
  //cvCvtColor(iplHandle, iplHandleC, CV_RGB2BGR);
  return iplHandle;
}

IplImage* ImageHandle::getOrigIplHandle() {
  if(!dataset && !rawData)
    return NULL;
  IplImage* iplHandle = cvCreateImage(cvSize(getWidth(), getHeight()), IPL_DEPTH_8U, 3);
  unsigned char* rData;
  //if(dataset) {
  //  rData = copyRawImageData();
  //}
  //else {
    rData = origData;
    //}
  int width = getWidth();
  int height = getHeight();

  unsigned char* rawData2 = new unsigned char[width*height*3];
  unsigned char* rp = rData;
  for(unsigned int i = 0; i < height; i++, rp += width * 3) {
    for(unsigned int k = 0; k < width; k++) {
      rawData2[(i*width*3) + 3 * k] = rp[3 * k + 2];
      rawData2[(i*width*3) + 3 * k + 1] = rp[3 * k + 1];
      rawData2[(i*width*3) + 3 * k + 2] = rp[3 * k];
    }
  }

  cvSetData(iplHandle, rawData2, getWidth() * 3);
  //IplImage* iplHandeC = cvCloneImage(iplHandle);
  //cvCvtColor(iplHandle, iplHandleC, CV_RGB2BGR);
  return iplHandle;
}

unsigned char* ImageHandle::copyRawImageData() {
  if(!dataset) {
    if(!rawData)
      return NULL;
  }
  
  if(rawData) {
    int amount = width * height * 3;
    unsigned char* imData = new unsigned char[amount];
    memcpy(imData, rawData, amount);
    return imData;
  }
  int width, height, bpp;

  width = getWidth();
  height = getHeight();
  bpp = getBpp();

  unsigned char* imageData = new unsigned char[width * height * bpp];
  unsigned char* buf = new unsigned char[width * height * bpp];
  
  CPLErr retVal = dataset->RasterIO(GF_Read, 0, 0, width, height, imageData, width, height, GDT_Byte, dataset->GetRasterCount(), NULL, 0, 0, 0);

  if(retVal == CE_Failure) {
    string error;
    error = "Error while loading image: ";
    error += imagePath;
    logError(error, 3);

  }

  if(dataset->GetRasterCount() == 3) {
    for(int i = 0; i < bpp; i++)
      for(int k = 0; k < width * height; k++) { 
	buf[(k * bpp) + i] = imageData[(i * (height * width)) + k] ;
      }
  }
  else {
    for(int k = 0; k < width * height; k++) { 
	buf[3 * k] = imageData[k] ;
	buf[3 * k + 1] = imageData[k] ;
	buf[3 * k + 2] = imageData[k] ;
      }
  }
  delete[] imageData;
  return buf;
}

int ImageHandle::clonePhysical(string path, string type, ImageHandle** newHandle) {
  GDALDriverManager* mgr = GetGDALDriverManager();
  GDALDriver* selDriver = mgr->GetDriverByName(type.c_str());
  if(!selDriver) {
    return -1;
  }

  if(dataset) {
    GDALDataset* newDataset = selDriver->CreateCopy(path.c_str(), dataset, FALSE, NULL, NULL, NULL);
  }
  else {
    dataset = selDriver->Create(path.c_str(), getWidth(), getHeight(), 3, GDT_Byte, NULL);
    int bandList[3] = {1, 2, 3};
    dataset->RasterIO(GF_Write, 0, 0, getWidth(), getHeight(), rawData, getWidth() , getHeight() , GDT_Byte, 3, bandList, 3, getWidth() * 3, 1);
    dataset->FlushCache();
  }
}

void ImageHandle::getRawData(unsigned char** data, int *step, Roi* roi) {
  if(data) {
    *data = rawData;
  }
  if(step)
    *step = getWidth() * 3;
}

unsigned char* ImageHandle::getOriginalRawData() {
  return origData;
}

unsigned char* ImageHandle::copyOriginalRawData() {
  unsigned char* origData2 = new unsigned char[getWidth() * getHeight() * 3];
  memcpy(origData2, origData, sizeof(unsigned char) * getWidth() * getHeight() * 3);
  return origData2;
}
