#include <xercesc/dom/DOM.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/framework/LocalFileFormatTarget.hpp>

#include <iostream>
#include <fstream>
#include "projectFactory.h"
#include "initializers.h"

using namespace std;

string projectAddress;

Content* ProjectSerializer::parseContentNode(DOMNode* node) {
	string comment;
	string address;
	string name;
	string assocCamera;
	string worldFile;
	float phi, omega, kappa, x, y, z, px, py, pSize, f;
	string assocDem;

	ContentType contentType;
	char* rootNodeName = XMLString::transcode(node->getNodeName());
	if(stricmp(rootNodeName, "content"))
		return NULL;

	XMLString::release(&rootNodeName);

	DOMNode* currentChild = node->getFirstChild();

	DOMNamedNodeMap* attMap;
	attMap = node->getAttributes();
	if(attMap) {
		DOMNode* typeNode = attMap->getNamedItem(XMLString::transcode("type"));
		contentType = convertToContentType(XMLString::transcode(typeNode->getNodeValue()));
	}
	else {
		contentType = Image2D;
	}

	while(currentChild) {
		char* nodeName = XMLString::transcode(currentChild->getNodeName());
		char* textContent = XMLString::transcode(currentChild->getTextContent());

		if(!stricmp(nodeName, "comment"))
			comment = textContent;
		if(!stricmp(nodeName, "address"))
		        address = textContent;
		if(!stricmp(nodeName, "name"))
			name = textContent;
		if(!stricmp(nodeName, "worldFile")) 
		  worldFile = textContent;
		if(!stricmp(nodeName, "f"))
		  f = atof(textContent);
		if(!stricmp(nodeName, "x"))
		  x = atof(textContent);
		if(!stricmp(nodeName, "y"))
		  y = atof(textContent);
		if(!stricmp(nodeName, "z"))
		  z = atof(textContent);
		if(!stricmp(nodeName, "px"))
		  px = atof(textContent);
		if(!stricmp(nodeName, "py"))
		  py = atof(textContent);
		if(!stricmp(nodeName, "pSize"))
		  pSize = atof(textContent);
		if(!stricmp(nodeName, "phi"))
		  phi = atof(textContent);
		if(!stricmp(nodeName, "kappa"))
		  kappa = atof(textContent);
		if(!stricmp(nodeName, "omega"))
		  omega = atof(textContent);
		if(!stricmp(nodeName, "assocCamera"))
		  assocCamera = textContent;
		if(!stricmp(nodeName, "assocDem"))
		  assocDem = textContent;
		currentChild = currentChild->getNextSibling();
		XMLString::release(&nodeName);
		XMLString::release(&textContent);
	}
	Content* newContent;
	Camera* newContent2;
	Image* newContent3;
	switch(contentType) {
	case Image2D:
	  newContent = new Image(name, address, comment, worldFile);		
	  newContent3 = static_cast<Image*>(newContent);
	  newContent3->setCamera(assocCamera);
	  newContent3->setDem(assocDem);
	  break;
	case CameraType:
	  newContent = new Camera(name, comment);
	  newContent2 = static_cast<Camera*>(newContent);
	  newContent2->f = f;
	  newContent2->x = x;
	  newContent2->y = y;
	  newContent2->z = z;
	  newContent2->px = px;
	  newContent2->py = py;
	  newContent2->pSize = pSize;
	  newContent2->phi = phi;
	  newContent2->kappa = kappa;
	  newContent2->omega = omega;
	  break;
	case DEM:
	  newContent = loadDem(projectAddress.substr(0, projectAddress.find_last_of("\\/") + 1) + address, name, comment);
	  break;
	case VideoType:
	  newContent = new Video(name, address, comment);
	  break;
	default:
	  newContent = new Content(name, address, comment, contentType);
	  break;
	}
	return newContent;
}
Project* ProjectSerializer::loadProject(string projectPath) {
  projectAddress = projectPath;
	vector<string> fileList;
	Project* loadedProject = new Project;

	XercesDOMParser* projectDOM = getProjectDOM(projectPath);

	xercesDOMDocument* from = projectDOM->getDocument();

	try {
		DOMElement* rootElement = from->getDocumentElement();
		DOMNode* currentChild = rootElement->getFirstChild();

		while(currentChild) {
			char* fileName = XMLString::transcode(currentChild->getTextContent());
			char* nodeName = XMLString::transcode(currentChild->getNodeName());
			Content* newContent;
			Meta* metaContent;
			if(!stricmp(nodeName, "content")) {
				newContent = parseContentNode(currentChild);
				loadedProject->addContent(newContent);
			}
			if(!stricmp(nodeName, "meta")) {
				metaContent = parseMetaNode(currentChild);
				if(metaContent) {
					loadedProject->setMetaData(metaContent);
					delete metaContent;
				}
			}

			currentChild = currentChild->getNextSibling();
		}
	}
	catch(XMLException& toCatch) {
		char* errorMessage = XMLString::transcode(toCatch.getMessage());
		cout << "Error while parsing the project file!" << endl <<
			"Error message starts here-->" << endl << errorMessage <<
			"-->Error message ends here" << endl;
		XMLString::release(&errorMessage);
		return NULL;
	}
	catch(DOMException& domException) {
		char* errorMessage = XMLString::transcode(domException.getMessage());
		cerr << "Error while parsing the project file!" << endl <<
			"Error message starts here-->" << endl << errorMessage <<
			"-->Error message ends here" << endl;
		XMLString::release(&errorMessage);
		return NULL;
	}
	catch(...) {
		cerr << "Unknown error in parsing the project xml file!" << endl;
		return NULL;
	}
#ifdef LINUX
	loadedProject->address = projectPath.substr(0, projectPath.find_last_of("/"));
#elif WIN32
	loadedProject->address = projectPath.substr(0, projectPath.find_last_of("\\"));#
#endif
	return loadedProject;
}

XercesDOMParser* ProjectSerializer::getProjectDOM(string projectPath) {
		XercesDOMParser* projectParser;
		try {

	 projectParser = new XercesDOMParser();
		projectParser->setValidationScheme(XercesDOMParser::Val_Never);
		projectParser->setDoNamespaces(false);

		ErrorHandler* errHandler = (ErrorHandler*) new HandlerBase();
		projectParser->setErrorHandler(errHandler);


		projectParser->parse(projectPath.c_str());
	}
	catch(const XMLException& toCatch) {
		char* errorMessage = XMLString::transcode(toCatch.getMessage());
		cerr << "Error while parsing the project file!" << endl <<
			"Error log starts here-->" << errorMessage << endl <<
			"<--Error log ends here" << endl;
		XMLString::release(&errorMessage);
		return NULL;
	}
	catch(const DOMException& toCatch) {
		char* errorMessage = XMLString::transcode(toCatch.getMessage());
		cerr << "Error while parsing the project file!" << endl <<
			"Error log starts here-->" << errorMessage << endl <<
			"<--Error log ends here" << endl;
		XMLString::release(&errorMessage);
		return NULL;
	}
	catch(...) {
		cerr << "Unexpected exception received while parsing project file." << endl;
		return NULL;
	}

	return projectParser;
}
Meta* ProjectSerializer::parseMetaNode(DOMNode* node) {
	Meta* metaData = NULL;
	string projectName;

	char* rootNodeName = XMLString::transcode(node->getNodeName());
	if(stricmp(rootNodeName, "meta"))
		return NULL;

	XMLString::release(&rootNodeName);

	DOMNode* currentChild = node->getFirstChild();

	while(currentChild) {
		char* nodeName = XMLString::transcode(currentChild->getNodeName());
		char* textContent = XMLString::transcode(currentChild->getTextContent());

		if(!stricmp(nodeName, "projectName"))
			projectName = textContent;
		currentChild = currentChild->getNextSibling();
		XMLString::release(&nodeName);
		XMLString::release(&textContent);
	}
	metaData = new Meta(projectName);
	return metaData;
}
void ProjectSerializer::normalizeImageAddresses(Project* project) {
  vector<Content*>* contents = project->getContents();
  Image* currentImage;

  for(unsigned int i = 0; i < contents->size(); i++) {
    if((*contents)[i]->contentType == Image2D) { // ||
       //((*contents)[i]->contentType == Orthophoto) ||
       //((*contents)[i]->contentType == Mosaick)) {
      currentImage = static_cast<Image*>((*contents)[i]);
      if(currentImage->addressType == RelativeAddress) {
#ifdef LINUX
	currentImage->address = project->address + "/" + currentImage->address;
#elif WIN32
	currentImage->address = project->address + "\\" + currentImage->address;
#endif
      }
    }
    if((*contents)[i]->contentType == VideoType) {
      Content* content = (*contents)[i];
#ifdef LINUX
      content->address = project->address + "/" + content->address;
#elif WIN32
      content->address = project->address + "\\" + content->address;
#endif
    }
  }
}

Project* ProjectSerializer::deserializeProject(string projectPath) {
  Project* loadedProject =  loadProject(projectPath);
  normalizeImageAddresses(loadedProject);
  return loadedProject;
}


int ProjectSerializer::createMetaNode(DOMDocument* doc, DOMElement* rootNode, Meta* metaData) {
  XMLCh* tempStr;
  tempStr = XMLString::transcode("meta");
  DOMElement* metaNode = doc->createElement(tempStr);
  rootNode->appendChild(metaNode);
  
  tempStr = XMLString::transcode("name");
  DOMElement* nameNode = doc->createElement(tempStr);
  metaNode->appendChild(nameNode);

  tempStr = XMLString::transcode(metaData->getName().c_str());
  DOMText* nameText = doc->createTextNode(tempStr);
  nameNode->appendChild(nameText);
  
  return 1;
  
}

int ProjectSerializer::createContentNode(DOMDocument* doc, DOMElement* rootNode, Content* content) {
  XMLCh tempStr[1024];
  XMLString::transcode("content", tempStr, 1023);
  DOMElement* contentNode = doc->createElement(tempStr);
  rootNode->appendChild(contentNode);

  switch(content->contentType) {
  case Image2D:
    XMLString::transcode("image", tempStr, 1023);
    break;
  case Mosaic:
    XMLString::transcode("mosaic", tempStr, 1023);
    break;
  case DEM:
    XMLString::transcode("dem", tempStr, 1023);
    break;
  case OrthophotoType:
    XMLString::transcode("orthophoto", tempStr, 1023);
    break;
  case SuperResolution:
    XMLString::transcode("superresolution", tempStr, 1023);
    break;
  case CameraType:
    XMLString::transcode("camera", tempStr, 1023);
    break;
  }
  XMLCh tempStr2[1024];
  XMLString::transcode("type", tempStr2, 1023);

  contentNode->setAttribute(tempStr2, tempStr);

  XMLString::transcode("name", tempStr, 1023);
  DOMElement* nameNode = doc->createElement(tempStr);
  contentNode->appendChild(nameNode);
  XMLString::transcode(content->name.c_str(), tempStr, 1023);
  DOMText* nameText = doc->createTextNode(tempStr);
  nameNode->appendChild(nameText);
  
  if(content->contentType == Image2D ||
     content->contentType == Mosaic ||
     content->contentType == OrthophotoType ||
     content->contentType == DEM) {
    XMLString::transcode("address", tempStr, 1023);
    DOMElement* addressNode = doc->createElement(tempStr);
    contentNode->appendChild(addressNode);
    XMLString::transcode(content->address.c_str(), tempStr, 1023);
    DOMText* addressText = doc->createTextNode(tempStr);
    addressNode->appendChild(addressText);
  }

  XMLString::transcode("comment", tempStr, 1023);
  DOMElement* commentNode = doc->createElement(tempStr);
  contentNode->appendChild(commentNode);
  XMLString::transcode(content->comment.c_str(), tempStr, 1023);
  DOMText* commentText = doc->createTextNode(tempStr);
  commentNode->appendChild(commentText);

  if(content->contentType == CameraType) {
    double data;
    char buf[512];
    Camera* camera = (Camera*)content;

    data = camera->x;
    sprintf(buf, "%lf", data);
    XMLString::transcode("x", tempStr, 1023);
    DOMElement* xNode = doc->createElement(tempStr);
    contentNode->appendChild(xNode);
    XMLString::transcode(buf, tempStr, 1023);
    DOMText* xText = doc->createTextNode(tempStr);
    xNode->appendChild(xText);

    data = camera->y;
    sprintf(buf, "%lf", data);
    XMLString::transcode("y", tempStr, 1023);
    DOMElement* yNode = doc->createElement(tempStr);
    contentNode->appendChild(yNode);
    XMLString::transcode(buf, tempStr, 1023);
    DOMText* yText = doc->createTextNode(tempStr);
    yNode->appendChild(yText);

    data = camera->z;
    sprintf(buf, "%lf", data);
    XMLString::transcode("z", tempStr, 1023);
    DOMElement* zNode = doc->createElement(tempStr);
    contentNode->appendChild(zNode);
    XMLString::transcode(buf, tempStr, 1023);
    DOMText* zText = doc->createTextNode(tempStr);
    zNode->appendChild(zText);

    data = camera->f;
    sprintf(buf, "%lf", data);
    XMLString::transcode("f", tempStr, 1023);
    DOMElement* fNode = doc->createElement(tempStr);
    contentNode->appendChild(fNode);
    XMLString::transcode(buf, tempStr, 1023);
    DOMText* fText = doc->createTextNode(tempStr);
    fNode->appendChild(fText);
    
    data = camera->px;
    sprintf(buf, "%lf", data);
    XMLString::transcode("px", tempStr, 1023);
    DOMElement* pxNode = doc->createElement(tempStr);
    contentNode->appendChild(pxNode);
    XMLString::transcode(buf, tempStr, 1023);
    DOMText* pxText = doc->createTextNode(tempStr);
    pxNode->appendChild(pxText);

    data = camera->py;
    sprintf(buf, "%lf", data);
    XMLString::transcode("py", tempStr, 1023);
    DOMElement* pyNode = doc->createElement(tempStr);
    contentNode->appendChild(pyNode);
    XMLString::transcode(buf, tempStr, 1023);
    DOMText* pyText = doc->createTextNode(tempStr);
    pyNode->appendChild(pyText);

    data = camera->pSize;
    sprintf(buf, "%lf", data);
    XMLString::transcode("pSize", tempStr, 1023);
    DOMElement* pSizeNode = doc->createElement(tempStr);
    contentNode->appendChild(pSizeNode);
    XMLString::transcode(buf, tempStr, 1023);
    DOMText* pSizeText = doc->createTextNode(tempStr);
    pSizeNode->appendChild(pSizeText);

    data = camera->omega;
    sprintf(buf, "%lf", data);
    XMLString::transcode("omega", tempStr, 1023);
    DOMElement* omegaNode = doc->createElement(tempStr);
    contentNode->appendChild(omegaNode);
    XMLString::transcode(buf, tempStr, 1023);
    DOMText* omegaText = doc->createTextNode(tempStr);
    omegaNode->appendChild(omegaText);

    data = camera->phi;
    sprintf(buf, "%lf", data);
    XMLString::transcode("phi", tempStr, 1023);
    DOMElement* phiNode = doc->createElement(tempStr);
    contentNode->appendChild(phiNode);
    XMLString::transcode(buf, tempStr, 1023);
    DOMText* phiText = doc->createTextNode(tempStr);
    phiNode->appendChild(phiText);

    data = camera->kappa;
    sprintf(buf, "%lf", data);
    XMLString::transcode("kappa", tempStr, 1023);
    DOMElement* kappaNode = doc->createElement(tempStr);
    contentNode->appendChild(kappaNode);
    XMLString::transcode(buf, tempStr, 1023);
    DOMText* kappaText = doc->createTextNode(tempStr);
    kappaNode->appendChild(kappaText);

  }
  return 1;  
}

int ProjectSerializer::serializeDOMToFile(string projectPath, DOMNode* rootNode) {
  XMLCh tempStr[100];
  XMLString::transcode("LS", tempStr, 99);
  DOMImplementation *impl = DOMImplementationRegistry::getDOMImplementation(tempStr);
  DOMWriter* serializer  = ((DOMImplementationLS*)impl)->createDOMWriter();

  XMLFormatTarget* formTarget = new LocalFileFormatTarget(projectPath.c_str());

  if (serializer->canSetFeature(XMLUni::fgDOMWRTFormatPrettyPrint, true))
    serializer->setFeature(XMLUni::fgDOMWRTFormatPrettyPrint, true);

  serializer->writeNode(formTarget, *rootNode);
  formTarget->flush();
  return 1;

}

int ProjectSerializer::saveProject(Project* project, string projectPath) {
  XMLCh tempStr[100];

  XMLString::transcode("xml 1.0", tempStr, 99);
  DOMImplementation* impl = DOMImplementationRegistry::getDOMImplementation(tempStr);

  XMLString::transcode("project", tempStr, 99);
  DOMDocument* doc = impl->createDocument(0, tempStr, 0);
  DOMElement* root = doc->getDocumentElement();

  createMetaNode(doc, root, project->getMetaData());

  vector<Content*>* contents = project->getContents();
  for(vector<Content*>::iterator i = contents->begin(); i != contents->end(); i++) {
    createContentNode(doc, root, *i);
  }
  serializeDOMToFile(projectPath, root);
  return 1;
}

int ProjectSerializer::serializeProject(Project* project, string projectPath) {
  return saveProject(project, projectPath);
}

int ProjectSerializer::serializeOrthophoto(Orthophoto* op) {
  if(op->address == "INMEM")
    return -1;
  op->clonePhysical(op->address);
}

Dem* ProjectSerializer::loadDem(string demPath, string name, string comment) {
  ifstream input(demPath.c_str());
  int width, height;

  input >> width >> height;

  double* heightTable = new double[width * height];
  int val;
  for(unsigned int i = 0; i < width * height; i++) {
    input >> heightTable[i];
    //cout << heightTable[i] << " " ;
  }  
  //input.close();
  //cout << "Comment is: " << comment << endl << "Name is: " << name << endl << "Path is: " << demPath << endl; 
  Dem* newDem = new Dem(heightTable, width, height, name, demPath, comment);
  return newDem;
}
